#include <stdint.h>
#include <terminal.h>
#include <mittos/graphics.h>
#include <memory.h>

gfx_context kernel_fb;
gfx_context *term_fb;
static int setup = 0;

static void draw_border(gfx_context *ctx, uint32_t clr)
{
    for(uint64_t x = 0; x < ctx->width; x++)
    {
        putpixel(ctx, x, 0, clr);
        putpixel(ctx, x, ctx->height-1, clr);
    }
    for(uint64_t y = 0; y < ctx->height; y++)
    {
        putpixel(ctx, 0, y, clr);
        putpixel(ctx, ctx->width-1, y, clr);
    }
}

void fbterm_movecursor(unsigned int cursor)
{
    (void) cursor;
}

void fbterm_flush(struct vga_cell *buffer)
{
    if(!setup) return;
    int i = 0;
    for(int row = 0; row < VGA_ROWS; row++)
    {
        for(int col=0; col < VGA_COLS; col++)
        {
            putCharacter(term_fb, col*8, row*16, 0xFFFFFF, 0, (char)buffer[i++].c);
        }
    }
    flip(term_fb);
}

void fbterm_init(struct fbinfo *fbinfo)
{
    kernel_fb.width = fbinfo->framebuffer_width;
    kernel_fb.height = fbinfo->framebuffer_height;
    kernel_fb.bpp = fbinfo->framebuffer_bpp/8;
    kernel_fb.pitch = fbinfo->framebuffer_pitch;
    kernel_fb.addr = P2V(fbinfo->framebuffer_addr);
    kernel_fb.size = kernel_fb.pitch * (kernel_fb.height);
    kernel_fb.buffer = calloc(1, kernel_fb.size);

    for(uintptr_t p = (uintptr_t)kernel_fb.addr; p < ((uintptr_t)kernel_fb.addr + kernel_fb.size); p += PAGE_SIZE)
    {
        vmm_set_page(kernel_P4, p, V2P(p), PAGE_WRITE | PAGE_PRESENT);
    }

    term_fb = framebuffer_make_subcontext(&kernel_fb, 20, 20, 8*VGA_COLS+8, 16*VGA_ROWS+8);
    draw_border(term_fb, RGB(255, 0, 0));
    flip(term_fb);
    term_fb = framebuffer_make_subcontext(&kernel_fb, 24, 24, 8*VGA_COLS, 16*VGA_ROWS);
    setup = 1;
}